@@ -47,8 +47,6 @@ VAR omBin sip_package_bin = omGetSpecBin(sizeof(sip_package));
 //omBin ip_package_bin = omGetSpecBin(sizeof(ip_package));
 VAR omBin idrec_bin = omGetSpecBin(sizeof(idrec));
 
-VAR coeffs coeffs_BIGINT;
-
 VAR proclevel *procstack=NULL;
 //idhdl idroot = NULL;
 
@@ -78,7 +76,8 @@ idhdl idrec::get(const char * s, int level)
   int l;
   const char *id_;
   unsigned long i=iiS2I(s);
-  int less4=(i < (1L<<((SIZEOF_LONG-1)*8)));
+  char *dummy=(char*)&i;
+  BOOLEAN less4=(dummy[SIZEOF_LONG-1]=='\0');
   while (h!=NULL)
   {
     omCheckAddr((ADDRESS)IDID(h));
@@ -150,6 +149,8 @@ void *idrecDataInit(int t)
 #endif
     case BIGINTMAT_CMD:
       return (void *)new bigintmat();
+    case BIGINTVEC_CMD:
+      return (void *)new bigintmat(1,0,coeffs_BIGINT);
     case BUCKET_CMD:
       if (currRing!=NULL)
       return (void*)sBucketCreate(currRing);
@@ -313,7 +314,10 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
             Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
         if (s==IDID(h)) IDID(h)=NULL;
-        killhdl2(h,root,currRing);
+        if((t!=PROC_CMD)||(IDPROC(h)->language!=LANG_C))
+        {
+          killhdl2(h,root,currRing);
+        }
       }
     }
     else
@@ -335,6 +339,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
             Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
         if (s==IDID(h)) IDID(h)=NULL;
+        //  proc is not ring-dep, no need to check for type "proc":
         killhdl2(h,&currRing->idroot,currRing);
       }
       else
@@ -357,7 +362,10 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
             Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
         if (s==IDID(h)) IDID(h)=NULL;
-        killhdl2(h,&IDROOT,NULL);
+        if((t!=PROC_CMD)||(IDPROC(h)->language!=LANG_C))
+        {
+          killhdl2(h,&IDROOT,NULL);
+        }
       }
       else
         goto errlabel;
@@ -373,7 +381,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     //Werror("identifier `%s` in use(lev h=%d,typ=%d,t=%d, curr=%d)",s,IDLEV(h),IDTYP(h),t,lev);
     Werror("identifier `%s` in use",s);
     //listall();
-    omFree((ADDRESS)s);
+    omFreeBinAddr((ADDRESS)s);
     return NULL;
 }
 void killid(const char * id, idhdl * ih)
@@ -457,7 +465,8 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   }
   if (IDTYP(h) == PACKAGE_CMD)
   {
-    if (((IDPACKAGE(h)->language==LANG_C)&&(IDPACKAGE(h)->idroot!=NULL))
+    if ((((IDPACKAGE(h)->language==LANG_C) ||(IDPACKAGE(h)->language==LANG_MIX))
+      &&(IDPACKAGE(h)->idroot!=NULL))
     || (strcmp(IDID(h),"Top")==0))
     {
       Warn("cannot kill `%s`",IDID(h));
@@ -481,7 +490,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
         hdh = temp;
       }
       killhdl2(*hd,hd,NULL);
-      if (IDPACKAGE(h)->libname!=NULL) omFree((ADDRESS)(IDPACKAGE(h)->libname));
+      if (IDPACKAGE(h)->libname!=NULL) omFreeBinAddr((ADDRESS)(IDPACKAGE(h)->libname));
     }
     paKill(IDPACKAGE(h));
     if (currPackHdl==h) currPackHdl=packFindHdl(currPack);
@@ -491,10 +500,11 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
     rKill(h);
   else if (IDDATA(h)!=NULL)
     s_internalDelete(IDTYP(h),IDDATA(h),r);
+  IDDATA(h)=NULL;
   //  general  -------------------------------------------------------------
   // now dechain it and delete idrec
   if (IDID(h)!=NULL) // OB: ?????
-    omFree((ADDRESS)IDID(h));
+    omFreeBinAddr((ADDRESS)IDID(h));
   IDID(h)=NULL;
   IDDATA(h)=NULL;
   if (h == (*ih))
@@ -728,7 +738,7 @@ const char * piProcinfo(procinfov pi, const char *request)
   else if (strcmp(request, "ref")      == 0)
   {
     char p[8];
-    sprintf(p, "%d", pi->ref);
+    snprintf(p,8, "%d", pi->ref);
     return omStrDup(p);  // MEMORY-LEAK
   }
   return "??";
@@ -753,9 +763,9 @@ BOOLEAN piKill(procinfov pi)
       }
     }
     if (pi->libname != NULL) // OB: ????
-      omFree((ADDRESS)pi->libname);
+      omFreeBinAddr((ADDRESS)pi->libname);
     if (pi->procname != NULL) // OB: ????
-      omFree((ADDRESS)pi->procname);
+      omFreeBinAddr((ADDRESS)pi->procname);
 
     if( pi->language == LANG_SINGULAR)
     {
@@ -784,7 +794,7 @@ void paCleanUp(package pack)
       dynl_close (pack->handle);
 #endif /* HAVE_DYNAMIC_LOADING */
     }
-    omFree((ADDRESS)pack->libname);
+    omFreeBinAddr((ADDRESS)pack->libname);
     memset((void *) pack, 0, sizeof(sip_package));
     pack->language=LANG_NONE;
   }
@@ -876,8 +886,8 @@ BOOLEAN iiAlias(leftv p)
       case MAP_CMD:
          {
            map im = IDMAP(pp);
-           omFree((ADDRESS)im->preimage); 
-	   im->preimage=NULL;// and continue
+           omFreeBinAddr((ADDRESS)im->preimage);
+           im->preimage=NULL;// and continue
          }
          // continue as ideal:
       case IDEAL_CMD:
@@ -898,8 +908,16 @@ BOOLEAN iiAlias(leftv p)
          break;
        // case ring: cannot happen
        default:
-         Werror("unknown type %d",p->Typ());
-         return TRUE;
+         {
+           int t=p->Typ();
+           if (t<MAX_TOK)
+           {
+             Werror("unknown type %d(%s)",t,Tok2Cmdname(t));
+             return TRUE;
+           }
+           /*else: blackbox type, not yet set*/
+           break;
+         }
   }
   pp->typ=ALIAS_CMD;
   IDDATA(pp)=(char*)h->data;
