@@ -14,6 +14,7 @@
 #define PLURAL_INTERNAL_DECLARATIONS 1
 
 #include "kernel/mod2.h"
+#include "kernel/oswrapper/vspace.h"
 #include "misc/sirandom.h"
 #include "omalloc/omalloc.h"
 #include "misc/mylimits.h"
@@ -561,6 +562,7 @@ const struct soptionStruct verboseStruct[]=
   {"warn",     Sy_bit(V_ALLWARN),   ~Sy_bit(V_ALLWARN)},
   {"intersectSyz",Sy_bit(V_INTERSECT_SYZ), ~Sy_bit(V_INTERSECT_SYZ)},
   {"intersectElim",Sy_bit(V_INTERSECT_ELIM), ~Sy_bit(V_INTERSECT_ELIM)},
+  {"pure_gb",Sy_bit(V_PURE_GB), ~Sy_bit(V_PURE_GB)},
 /*special for "none" and also end marker for showOption:*/
   {"ne",         0,          0 }
 };
@@ -1136,10 +1138,6 @@ void m2_end(int i)
     }
 #endif   // HAVE_SIMPLEIPC
     monitor(NULL,0);
-#ifdef PAGE_TEST
-    mmEndStat();
-#endif
-    fe_reset_input_mode();
     if (ssiToBeClosed_inactive)
     {
       link_list hh=ssiToBeClosed;
@@ -1174,34 +1172,24 @@ void m2_end(int i)
         hh=ssiToBeClosed;
       }
     }
+#ifdef PAGE_TEST
+    mmEndStat();
+#endif
+    fe_reset_input_mode();
     if (!singular_in_batchmode)
     {
-      if (i<=0)
+      if (TEST_V_QUIET)
       {
-        //extern long all_farey;
-        //extern long farey_cnt;
-        //if (all_farey!=0L) printf("farey:%ld, cnt=%ld\n",all_farey,farey_cnt);
-        if (TEST_V_QUIET)
-        {
-          if (i==0)
-            printf("Auf Wiedersehen.\n");
-          else
-            printf("\n$Bye.\n");
-        }
-        //#ifdef sun
-        //  #ifndef __svr4__
-        //    _cleanup();
-        //    _exit(0);
-        //  #endif
-        //#endif
-        i=0;
+        if (i==0)
+          printf("Auf Wiedersehen.\n");
       }
-      else
+      if (i>0)
       {
         printf("\nhalt %d\n",i);
       }
     }
-    exit(i);
+    if (i>=0) exit(i);
+    else _exit(0);
   }
 }
 }
@@ -1429,13 +1417,15 @@ void siInit(char *name)
 #endif
 
 // random generator: -----------------------------------------------
-  int t=initTimer();
-  if (t==0) t=1;
-  initRTimer();
-  siSeed=t;
-  factoryseed(t);
-  siRandomStart=t;
-  feOptSpec[FE_OPT_RANDOM].value = (void*) ((long)siRandomStart);
+  {
+    int t=startTimer();
+    if (t==0) t=1;
+    initRTimer();
+    siSeed=t;
+    factoryseed(t);
+    siRandomStart=t;
+    feOptSpec[FE_OPT_RANDOM].value = (void*) ((long)siRandomStart);
+  }
 
 // ressource table: ----------------------------------------------------
   // Don't worry: ifdef OM_NDEBUG, then all these calls are undef'ed
@@ -1446,16 +1436,25 @@ void siInit(char *name)
   slStandardInit();
   myynest=0;
 // how many processes ? -----------------------------------------------------
-  int cpus=2;
-  int cpu_n;
-  #ifdef _SC_NPROCESSORS_ONLN
-  if ((cpu_n=sysconf(_SC_NPROCESSORS_ONLN))>cpus) cpus=cpu_n;
-  #elif defined(_SC_NPROCESSORS_CONF)
-  if ((cpu_n=sysconf(_SC_NPROCESSORS_CONF))>cpus) cpus=cpu_n;
-  #endif
-  feSetOptValue(FE_OPT_CPUS, cpus);
+  {
+    int cpus=2;
+    int cpu_n;
+    #ifdef _SC_NPROCESSORS_ONLN
+    if ((cpu_n=sysconf(_SC_NPROCESSORS_ONLN))>cpus) cpus=cpu_n;
+    #elif defined(_SC_NPROCESSORS_CONF)
+    if ((cpu_n=sysconf(_SC_NPROCESSORS_CONF))>cpus) cpus=cpu_n;
+    #endif
+    if (cpus>SIPC_MAX_SEMAPHORES) cpus=SIPC_MAX_SEMAPHORES;
+    char *env_cpu=getenv("SINGULAR_CPUS");
+    if (env_cpu!=NULL)
+    {
+      int t=atoi(env_cpu);
+      if ((t>=0)&&(t<cpus)) cpus=t;
+    }
+    feSetOptValue(FE_OPT_CPUS, cpus);
 // how many threads ? -----------------------------------------------------
-  feSetOptValue(FE_OPT_THREADS, cpus);
+    feSetOptValue(FE_OPT_THREADS, cpus);
+  }
 
 // default coeffs
   {
