@@ -90,20 +90,23 @@ char * newstruct_String(blackbox *b, void *d)
       StringAppendS(a->name);
       StringAppendS("=");
       if (((!RingDependend(a->typ)&&!RingDependend(l->m[a->pos].rtyp)))
-      || ((rEqual((ring)l->m[a->pos-1].data,currRing))
+      //|| ((rEqual((ring)l->m[a->pos-1].data,currRing))
+      || (((ring)l->m[a->pos-1].data==currRing)
          && (currRing!=NULL)))
       {
+        // list may depend on currRing
         if (l->m[a->pos].rtyp==LIST_CMD)
         {
           StringAppendS("<list>");
         }
-        else if (l->m[a->pos].rtyp==STRING_CMD)
+        else
+        if (l->m[a->pos].rtyp==STRING_CMD)
         {
           StringAppendS((char*)l->m[a->pos].Data());
         }
         else
         {
-          char *tmp2=omStrDup(l->m[a->pos].String());
+          char *tmp2=l->m[a->pos].String();
           if ((strlen(tmp2)>80)||(strchr(tmp2,'\n')!=NULL))
           {
             StringAppendS("<");
@@ -114,7 +117,12 @@ char * newstruct_String(blackbox *b, void *d)
           omFree(tmp2);
         }
       }
-      else StringAppendS("??");
+      else
+      {
+        StringAppendS("<");
+        StringAppendS(Tok2Cmdname(l->m[a->pos].rtyp));
+        StringAppendS(">");
+      }
       if (a->next==NULL) break;
       StringAppendS("\n");
       if(errorreported) break;
@@ -604,7 +612,7 @@ BOOLEAN newstruct_serialize(blackbox *b, void *d, si_link f)
   l.rtyp=INT_CMD;
   l.data=(void*)(long)Ll;
   f->m->Write(f, &l);
-  // set all entries corresponding to "real" mebers to 1 in rings
+  // set all entries corresponding to "real" members to 1 in rings
   char *rings=(char*)omAlloc0(Ll+1);
   newstruct_member elem=dd->member;
   while (elem!=NULL)
@@ -640,15 +648,18 @@ BOOLEAN newstruct_deserialize(blackbox **, void **d, si_link f)
   // rtyp must be set correctly (to the blackbox id) by routine calling
   // newstruct_deserialize
   leftv l=f->m->Read(f); // int: length of list
-  int Ll=(int)(long)(l->data);
+  int Ll=(int)(long)l->data;
   omFreeBin(l,sleftv_bin);
   lists L=(lists)omAllocBin(slists_bin);
   L->Init(Ll+1);
   for(int i=0;i<=Ll;i++)
   {
     l=f->m->Read(f);
-    memcpy(&(L->m[i]),l,sizeof(sleftv));
-    omFreeBin(l,sleftv_bin);
+    if (l!=NULL)
+    {
+      memcpy(&(L->m[i]),l,sizeof(sleftv));
+      omFreeBin(l,sleftv_bin);
+    }
   }
   //newstruct_desc n=(newstruct_desc)b->data;
   //TODO: check compatibility of list l->data with description in n
