@@ -5,26 +5,20 @@
 * ABSTRACT: resolutions
 */
 
-
-
-
-
-#include <kernel/mod2.h>
-#include <misc/options.h>
-#include <omalloc/omalloc.h>
-#include <kernel/polys.h>
-#include <kernel/GBEngine/kstd1.h>
-#include <kernel/GBEngine/kutil.h>
-#include <kernel/combinatorics/stairc.h>
-//#include "cntrlc.h"
-#include <misc/intvec.h>
-#include <coeffs/numbers.h>
-#include <kernel/ideals.h>
-#include <misc/intvec.h>
-#include <polys/monomials/ring.h>
-#include <kernel/GBEngine/syz.h>
-#include <polys/kbuckets.h>
-#include <polys/prCopy.h>
+#include "kernel/mod2.h"
+#include "misc/options.h"
+#include "kernel/polys.h"
+#include "kernel/GBEngine/kstd1.h"
+#include "kernel/GBEngine/kutil.h"
+#include "kernel/combinatorics/stairc.h"
+#include "misc/intvec.h"
+#include "coeffs/numbers.h"
+#include "kernel/ideals.h"
+#include "misc/intvec.h"
+#include "polys/monomials/ring.h"
+#include "kernel/GBEngine/syz.h"
+#include "polys/kbuckets.h"
+#include "polys/prCopy.h"
 
 static void syInitSort(ideal arg,intvec **modcomp)
 {
@@ -332,7 +326,7 @@ static ideal sySchreyersSyzygiesFM(ideal arg,intvec ** modcomp)
 */
 //Print("toRed now (neuer ecart %d): ",ecartToRed);pWrite(toRed);
           notFound = TRUE;
-          bestEcart = 32000;  //a very large integer
+          bestEcart = INT_MAX-2;  //a very large integer
           p = NULL;
           int l=0;
 #define OLD_SEARCH
@@ -595,7 +589,7 @@ static ideal sySchreyersSyzygiesFB(ideal arg,intvec ** modcomp,ideal mW,BOOLEAN
           syz->coef = nCopy(currRing->qideal->m[k-Fl]->coef);
           syz->coef = nInpNeg(syz->coef);
           lastmonom = syz;
-          multWith = pDivide(syz,F[j]);
+          multWith = pMDivide(syz,F[j]);
           multWith->coef = nCopy(currRing->qideal->m[k-Fl]->coef);
         }
         pSetComp(syz,j+1);
@@ -697,7 +691,7 @@ static ideal sySchreyersSyzygiesFB(ideal arg,intvec ** modcomp,ideal mW,BOOLEAN
 #ifdef WRITE_BUCKETS
               PrintS("multiplied with: ");nWrite(up);PrintLn();
 #endif
-              pMult_nn(syz,up);
+              syz=__p_Mult_nn(syz,up,currRing);
             }
             nDelete(&up);
 
@@ -929,7 +923,7 @@ resolvente sySchreyerResolvente(ideal arg, int maxlength, int * length,
       }
       else
       {
-        syRing = rAssure_SyzComp_CompLastBlock(origR, TRUE);
+        syRing = rAssure_SyzComp_CompLastBlock(origR);
         if (syRing != origR)
         {
           rChangeCurrRing(syRing);
@@ -1035,6 +1029,7 @@ syStrategy sySchreyer(ideal arg, int maxlength)
   {
     if (fr[i]!=NULL)
     {
+      idSkipZeroes(fr[i]);
       result->fullres[i] = fr[i];
       fr[i] = NULL;
     }
